from typing import (
    Callable,
    Dict,
    Generic,
    TypeVar,
)

from edpconfigprovider import ConfigProvider

T = TypeVar("T")


class ConfigMapper(Generic[T]):
    def __init__(
        self,
        config_provider: ConfigProvider,
        transform_method: Callable[[Dict], T],
    ) -> None:
        self.config_provider = config_provider
        self.transform_method = transform_method

    def build_config(self) -> T:
        configuration_data = self.config_provider.get_configuration()
        return self.transform_method(configuration_data)



import json
import logging
import urllib.request
from urllib.error import (
    HTTPError,
    URLError,
)

from edpconfigprovider import ConfigProvider
from edpconfigprovider.exceptions import NoConfigurationException

logger = logging.getLogger(__name__)


class AwsAppconfigAgentProvider(ConfigProvider):
    def __init__(
        self,
        application_name: str,
        profile_name: str,
        environment_name: str,
        port: int,
    ):
        self.application_name = application_name
        self.profile_name = profile_name
        self.environment_name = environment_name
        self.port = port  #os.environ.get("AWS_APPCONFIG_EXTENSION_HTTP_PORT", "2772")

    def get_configuration(self) -> dict:
        try:
            url = f"http://localhost:{self.port}/applications/{self.application_name}/environments/{self.environment_name}/configurations/{self.profile_name}"  # noqa
            return json.loads(urllib.request.urlopen(url).read())
        except (HTTPError, URLError):
            logger.exception("Failed to fetch configuration from AWS AppConfig")

            config_name = f"{self.application_name}/{self.environment_name}/{self.profile_name}"
            raise NoConfigurationException(configuration_name=config_name)



import json
import logging

import boto3
from botocore.exceptions import ClientError

from edpconfigprovider import ConfigProvider
from edpconfigprovider.exceptions import NoConfigurationException


logger = logging.getLogger(__name__)


class AwsSsmProvider(ConfigProvider):

    _CLIENT = None

    def __init__(
        self,
        parameter_path: str = '/',
        recursive: bool = True,
        split_path: bool = True,
        decrypt: bool = True,
    ):
        self.parameter_path = parameter_path
        self.recursive = recursive
        self.split_path = split_path
        self.decrypt = decrypt

    @classmethod
    def get_ssm_client(cls):
        if cls._CLIENT is None:
            cls._CLIENT = boto3.client("ssm")
        return cls._CLIENT

    def get_configuration(self) -> dict:
        try:
            client = self.get_ssm_client()
            paginator = client.get_paginator("get_parameters_by_path")
            pages = paginator.paginate(
                Path=self.parameter_path,
                Recursive=self.recursive,
                WithDecryption=self.decrypt,
            )

            configuration = {}
            for page in pages:
                for parameter in page["Parameters"]:
                    name = parameter["Name"]
                    key = name[len(self.parameter_path):].lstrip("/")
                    value = parameter["Value"]

                    if parameter.get("Type") == "StringList":
                        value = value.split(",")
                    else:
                        try:
                            value = json.loads(value)
                        except (json.JSONDecodeError, TypeError):
                            pass
                    if self.split_path:
                        parts = key.split("/")
                        target = configuration
                        for part in parts[:-1]:
                            existing = target.get(part)
                            if isinstance(existing, dict):
                                target = existing
                            else:
                                if existing is not None:
                                    logger.warning(
                                        "SSM parameter '%s/%s' scalar value will be"
                                        " overwritten by a nested dict (from key '%s')",
                                        self.parameter_path.rstrip("/"),
                                        part,
                                        name,
                                    )
                                target[part] = {}
                                target = target[part]
                        target[parts[-1]] = value
                    else:
                        configuration[key] = value

            if not configuration:
                raise NoConfigurationException(
                    configuration_name=self.parameter_path
                )

            return configuration

        except ClientError:
            raise NoConfigurationException(
                configuration_name=self.parameter_path
            )



from threading import current_thread
from typing import (
    Dict,
    TypeVar,
)

from edpconfigprovider import ConfigProvider
from edpconfigprovider.builder import ProviderBuilder

T = TypeVar("T")


class ByTenantDynamicConfigProvider(ConfigProvider):
    def __init__(
        self,
        provider_builder: ProviderBuilder,
    ) -> None:
        self.provider_builder = provider_builder

        self._built_config_providers: Dict[str, ConfigProvider] = {}

    def get_configuration(self) -> dict:
        thread = current_thread()
        tenant_name = getattr(thread, "_edp_tenant_name", None)
        if tenant_name is None:
            tenant_name = getattr(thread, "_edp_context_name", None)
        if tenant_name is not None:
            tenant_name = tenant_name.split(".")[0]
        return self._build_config(tenant_name)

    def _build_config(self, tenant_name) -> dict:
        if tenant_name is None:
            return {}

        if tenant_name not in self._built_config_providers:
            self._built_config_providers[tenant_name] = self.provider_builder.build_provider(tenant_name=tenant_name)

        return {
            "name": tenant_name,
            "value": self._built_config_providers[tenant_name].get_configuration(),
        }
